#!/usr/bin/python
#
# Copyright INRA-URGI 2009-2013
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
import sys, os.path
from optparse import OptionParser
from commons.core.writer.FastqWriter import FastqWriter
from SMART.Java.Python.structure.Sequence import Sequence
from SMART.Java.Python.misc.UnlimitedProgress import UnlimitedProgress

class SamToFastq(object):

	def __init__(self, verbosity = 0):
		self.verbosity   = verbosity
		self.inputHandle = sys.stdin
		self.paired	     = False

	def setInputFile(self, fileName):
		if fileName != None:
			self.inputHandle = open(fileName)

	def setOutputFile(self, fileName):
		self.outputFileName = fileName

	def setPaired(self, boolean):
		self.paired = boolean

	def read(self):
		progress	 = UnlimitedProgress(10000, "Reading input file", self.verbosity)
		previousName = None
		for line in self.inputHandle:
			if line and line[0] != "@":
				splittedLine = line.strip().split()
				name	 = splittedLine[0]
				flags	= int(splittedLine[1])
				sequence = splittedLine[9]
				quality  = splittedLine[10]
				processedSequence = Sequence(name, sequence)
				processedSequence.setQuality(quality)
				if (flags & 0x10) != 0:
					processedSequence.reverseComplement()
				if self.paired:
					if previousName == name:
						self.outputHandles[0].addSequence(previousSequence)
						self.outputHandles[1].addSequence(processedSequence)
						name = None
					else:
						previousSequence = processedSequence
				else:
					self.outputHandle.addSequence(processedSequence)
				previousName = name
			progress.inc()
		progress.done()
		self.inputHandle.close()
		if self.paired:
			self.outputHandles[0].close()
			self.outputHandles[1].close()
		else:
			self.outputHandle.close()

	def run(self):
		if self.paired:
			self.outputFileName = os.path.splitext(self.outputFileName)[0]
			self.outputHandles = [FastqWriter("%s_1.fastq" % (self.outputFileName), self.verbosity), FastqWriter("%s_2.fastq" % (self.outputFileName), self.verbosity)]
		else:
			self.outputHandle = FastqWriter(self.outputFileName, self.verbosity)
		self.read()


if __name__ == "__main__":
	
	# parse command line
	description = "Sam To Fastq v1.0.0: Converts back a SAM file into a FASTQ file (no input file is STDIN). [Category: Personal]"

	parser = OptionParser(description = description)
	parser.add_option("-i", "--input",     dest="inputFileName",  action="store",      default=None,  type="string", help="input file [format: file in SAM format]")
	parser.add_option("-o", "--output",    dest="outputFileName", action="store",                     type="string", help="output file [compulsory] [format: output file in FASTQ format]")
	parser.add_option("-p", "--paired",    dest="paired",         action="store_true", default=False,                help="output file [compulsory] [format: output file in FASTQ format]")
	parser.add_option("-v", "--verbosity", dest="verbosity",      action="store",      default=1,     type="int",    help="trace level [format: int] [default: 1]")
	(options, args) = parser.parse_args()

	stf = SamToFastq(options.verbosity)
	stf.setInputFile(options.inputFileName)
	stf.setOutputFile(options.outputFileName)
	stf.setPaired(options.paired)
	stf.run()
